# Orca
#
# Copyright 2026 Igalia, S.L.
# Author: Joanmarie Diggs <jdiggs@igalia.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
# Boston MA  02110-1301 USA.

"""Command definitions for math navigation."""

from __future__ import annotations

from functools import partial
from typing import TYPE_CHECKING

from . import cmdnames, keybindings
from .command import Command, KeyboardCommand

if TYPE_CHECKING:
    from .math_navigator import MathNavigator


PLACE_MARKER_PREFIXES = ("MoveTo", "SetPlacemarker", "Read", "Describe")

COMMAND_SPECS = (
    ("math_move_next", "MoveNext", cmdnames.MATH_NAV_MOVE_NEXT, "Right", "none"),
    ("math_move_previous", "MovePrevious", cmdnames.MATH_NAV_MOVE_PREVIOUS, "Left", "none"),
    ("math_zoom_in", "ZoomIn", cmdnames.MATH_NAV_ZOOM_IN, "Down", "none"),
    ("math_zoom_out", "ZoomOut", cmdnames.MATH_NAV_ZOOM_OUT, "Up", "none"),
    ("math_zoom_in_all", "ZoomInAll", cmdnames.MATH_NAV_ZOOM_IN_ALL, "Down", "ctrl_shift"),
    ("math_zoom_out_all", "ZoomOutAll", cmdnames.MATH_NAV_ZOOM_OUT_ALL, "Up", "ctrl_shift"),
    ("math_move_start", "MoveStart", cmdnames.MATH_NAV_MOVE_START, "Home", "none"),
    ("math_move_end", "MoveEnd", cmdnames.MATH_NAV_MOVE_END, "End", "none"),
    ("math_move_line_start", "MoveLineStart", cmdnames.MATH_NAV_MOVE_LINE_START, "Home", "ctrl"),
    ("math_move_line_end", "MoveLineEnd", cmdnames.MATH_NAV_MOVE_LINE_END, "End", "ctrl"),
    (
        "math_move_column_start",
        "MoveColumnStart",
        cmdnames.MATH_NAV_MOVE_COLUMN_START,
        "Home",
        "shift",
    ),
    (
        "math_move_column_end",
        "MoveColumnEnd",
        cmdnames.MATH_NAV_MOVE_COLUMN_END,
        "End",
        "shift",
    ),
    (
        "math_move_cell_previous",
        "MoveCellPrevious",
        cmdnames.MATH_NAV_MOVE_CELL_PREVIOUS,
        "Left",
        "ctrl",
    ),
    (
        "math_move_cell_next",
        "MoveCellNext",
        cmdnames.MATH_NAV_MOVE_CELL_NEXT,
        "Right",
        "ctrl",
    ),
    ("math_move_cell_up", "MoveCellUp", cmdnames.MATH_NAV_MOVE_CELL_UP, "Up", "ctrl"),
    (
        "math_move_cell_down",
        "MoveCellDown",
        cmdnames.MATH_NAV_MOVE_CELL_DOWN,
        "Down",
        "ctrl",
    ),
    (
        "math_move_last_location",
        "MoveLastLocation",
        cmdnames.MATH_NAV_MOVE_LAST_LOCATION,
        "BackSpace",
        "none",
    ),
    (
        "math_toggle_zoom_lock_up",
        "ToggleZoomLockUp",
        cmdnames.MATH_NAV_TOGGLE_ZOOM_LOCK_UP,
        "Up",
        "shift",
    ),
    (
        "math_toggle_zoom_lock_down",
        "ToggleZoomLockDown",
        cmdnames.MATH_NAV_TOGGLE_ZOOM_LOCK_DOWN,
        "Down",
        "shift",
    ),
    (
        "math_toggle_speech_mode",
        "ToggleSpeakMode",
        cmdnames.MATH_NAV_TOGGLE_SPEECH_MODE,
        "space",
        "shift",
    ),
    ("math_read_previous", "ReadPrevious", cmdnames.MATH_NAV_READ_PREVIOUS, "Left", "shift"),
    ("math_read_next", "ReadNext", cmdnames.MATH_NAV_READ_NEXT, "Right", "shift"),
    ("math_read_current", "ReadCurrent", cmdnames.MATH_NAV_READ_CURRENT, "space", "none"),
    (
        "math_read_cell_current",
        "ReadCellCurrent",
        cmdnames.MATH_NAV_READ_CELL_CURRENT,
        "space",
        "ctrl",
    ),
    (
        "math_describe_previous",
        "DescribePrevious",
        cmdnames.MATH_NAV_DESCRIBE_PREVIOUS,
        "Left",
        "ctrl_shift",
    ),
    (
        "math_describe_next",
        "DescribeNext",
        cmdnames.MATH_NAV_DESCRIBE_NEXT,
        "Right",
        "ctrl_shift",
    ),
    (
        "math_describe_current",
        "DescribeCurrent",
        cmdnames.MATH_NAV_DESCRIBE_CURRENT,
        "space",
        "ctrl_shift",
    ),
    ("math_where_am_i", "WhereAmI", cmdnames.MATH_NAV_WHERE_AM_I, "", "unbound"),
    ("math_where_am_i_all", "WhereAmIAll", cmdnames.MATH_NAV_WHERE_AM_I_ALL, "", "unbound"),
    ("math_exit", "Exit", cmdnames.MATH_NAV_EXIT, "Escape", "none"),
    ("math_copy", "Copy", cmdnames.MATH_NAV_COPY, "", "unbound"),
)

MODIFIER_MASKS = {
    "none": keybindings.NO_MODIFIER_MASK,
    "ctrl": keybindings.CTRL_MODIFIER_MASK,
    "shift": keybindings.SHIFT_MODIFIER_MASK,
    "ctrl_shift": keybindings.CTRL_SHIFT_MODIFIER_MASK,
}


def _keybinding(key: str, modifier_name: str) -> keybindings.KeyBinding | None:
    """Returns the keybinding for key and modifier_name."""

    if modifier_name == "unbound":
        return None
    return keybindings.KeyBinding(key, MODIFIER_MASKS[modifier_name])


def get_supported_commands() -> list[str]:
    """Returns MathCAT navigation commands supported by the math navigator."""

    commands = [mathcat_command for _name, mathcat_command, _desc, _key, _mod in COMMAND_SPECS]
    for digit in range(10):
        commands.extend(f"{prefix}{digit}" for prefix in PLACE_MARKER_PREFIXES)
    return commands


def get_commands(owner: MathNavigator) -> list[Command]:
    """Returns commands for math navigation."""

    enter_binding = keybindings.KeyBinding("m", keybindings.ORCA_ALT_MODIFIER_MASK)
    commands: list[Command] = [
        KeyboardCommand(
            "math_enter",
            owner.enter_math_mode_command,
            owner.GROUP_LABEL,
            cmdnames.MATH_NAV_ENTER,
            desktop_keybinding=enter_binding,
            laptop_keybinding=enter_binding,
            is_group_toggle=True,
        )
    ]
    for name, mathcat_command, description, key, modifier_name in COMMAND_SPECS:
        if mathcat_command == "Exit":
            handler = owner.exit_math_mode
        elif mathcat_command == "Copy":
            handler = owner.copy_to_clipboard
        else:
            handler = partial(owner._execute_command, mathcat_command)  # pylint: disable=protected-access
        binding = _keybinding(key, modifier_name)
        commands.append(
            KeyboardCommand(
                name,
                handler,
                owner.GROUP_LABEL,
                description,
                desktop_keybinding=binding,
                laptop_keybinding=binding,
            )
        )

    marker_types = (
        ("none", PLACE_MARKER_PREFIXES[0], cmdnames.MATH_NAV_GOTO_PLACE_MARKER),
        ("ctrl", PLACE_MARKER_PREFIXES[1], cmdnames.MATH_NAV_SET_PLACE_MARKER),
        ("shift", PLACE_MARKER_PREFIXES[2], cmdnames.MATH_NAV_READ_PLACE_MARKER),
        (
            "ctrl_shift",
            PLACE_MARKER_PREFIXES[3],
            cmdnames.MATH_NAV_DESCRIBE_PLACE_MARKER,
        ),
    )
    for digit in ("1", "2", "3", "4", "5", "6", "7", "8", "9", "0"):
        for modifier_name, command_prefix, description in marker_types:
            binding = keybindings.KeyBinding(digit, MODIFIER_MASKS[modifier_name])
            commands.append(
                KeyboardCommand(
                    f"math_{command_prefix.lower()}_{digit}",
                    partial(owner._execute_command, f"{command_prefix}{digit}"),  # pylint: disable=protected-access
                    owner.GROUP_LABEL,
                    description,
                    desktop_keybinding=binding,
                    laptop_keybinding=binding,
                )
            )

    return commands



# Orca
#
# Copyright 2026 Igalia, S.L.
# Author: Joanmarie Diggs <jdiggs@igalia.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
# Boston MA  02110-1301 USA.

"""Base class for Orca extensions."""

from __future__ import annotations

import re
from dataclasses import dataclass
from typing import TYPE_CHECKING, Any, ClassVar

from . import command_manager, dbus_service, debug, gsettings_registry
from .extension_preferences import ExtensionPreference, ExtensionPreferenceKind

if TYPE_CHECKING:
    from .command import Command

__all__ = [
    "BrailleOutput",
    "BrailleOutputResult",
    "Extension",
    "ExtensionPreference",
    "ExtensionPreferenceKind",
    "SpeechOutput",
    "SpeechOutputResult",
]


@dataclass(frozen=True)
class SpeechOutput:
    """Text Orca is about to send to the active speech provider."""

    text: str
    obj: Any | None = None


@dataclass(frozen=True)
class SpeechOutputResult:
    """Result returned by an extension speech-output hook."""

    text: str | None = None
    consume: bool = False

    @classmethod
    def replace(cls, text: str) -> SpeechOutputResult:
        """Returns a result that replaces the text Orca will speak."""

        return cls(text=text)

    @classmethod
    def consume_output(cls) -> SpeechOutputResult:
        """Returns a result indicating that the extension handled speech output."""

        return cls(consume=True)


@dataclass(frozen=True)
class BrailleOutput:
    """Braille regions Orca is about to display."""

    regions: tuple[Any, ...] = ()
    focused_region: Any | None = None


@dataclass(frozen=True)
class BrailleOutputResult:
    """Result returned by an extension braille-output hook."""

    text: str | None = None
    consume: bool = False

    @classmethod
    def replace(cls, text: str) -> BrailleOutputResult:
        """Returns a result that replaces braille output with flat text."""

        return cls(text=text)

    @classmethod
    def consume_output(cls) -> BrailleOutputResult:
        """Returns a result indicating that the extension handled braille output."""

        return cls(consume=True)


class ExtensionSettings:
    """Settings scoped to one extension."""

    _SCHEMA = "extensions"
    _KEY = "settings"
    _SETTING_NAME = re.compile(r"^[A-Za-z0-9_.-]+$")

    def __init__(self, namespace: str) -> None:
        self._namespace = self._sanitize_namespace(namespace)

    @staticmethod
    def _sanitize_namespace(namespace: str) -> str:
        """Return a stable namespace for settings keys."""

        sanitized = gsettings_registry.GSettingsRegistry.sanitize_gsettings_path(namespace)
        return sanitized or "extension"

    def set_namespace(self, namespace: str) -> None:
        """Set the namespace used to store this extension's settings."""

        self._namespace = self._sanitize_namespace(namespace)

    def _sub_path(self) -> str:
        """Return the relocatable settings path for this extension."""

        return f"extensions/{self._namespace}"

    def _validate_key(self, key: str) -> str:
        """Return the storage key if it is valid."""

        if not key or not self._SETTING_NAME.fullmatch(key):
            raise ValueError(f"Invalid extension setting key: {key!r}")
        return key

    @classmethod
    def _unpack_setting_value(cls, value: Any) -> Any:
        """Return a plain Python value for an extension setting."""

        if isinstance(value, gsettings_registry.GLib.Variant):
            return cls._unpack_setting_value(value.unpack())
        if isinstance(value, dict):
            return {key: cls._unpack_setting_value(item) for key, item in value.items()}
        if isinstance(value, list):
            return [cls._unpack_setting_value(item) for item in value]
        return value

    def _get_local_settings(self) -> dict[str, Any]:
        """Return the active profile's extension settings."""

        registry = gsettings_registry.get_registry()
        gs = registry.get_settings(
            self._SCHEMA,
            registry.get_active_profile(),
            self._sub_path(),
        )
        if gs is None:
            return {}
        variant = gs.get_user_value(self._KEY)
        if variant is None:
            return {}
        value = self._unpack_setting_value(variant.unpack())
        return dict(value) if isinstance(value, dict) else {}

    def get(self, key: str, default: Any = None) -> Any:
        """Return a setting value using Orca's layered settings lookup."""

        settings = self._unpack_setting_value(
            gsettings_registry.get_registry().layered_lookup(
                self._SCHEMA,
                self._KEY,
                "a{sv}",
                sub_path=self._sub_path(),
                default={},
            )
        )
        if not isinstance(settings, dict):
            return default
        return settings.get(self._validate_key(key), default)

    def set(self, key: str, value: Any) -> bool:
        """Persist a setting value in the active profile."""

        settings = self._get_local_settings()
        settings[self._validate_key(key)] = value
        return gsettings_registry.get_registry().set_dict(
            self._SCHEMA,
            self._KEY,
            "a{sv}",
            settings,
            sub_path=self._sub_path(),
        )

    def reset(self, key: str) -> bool:
        """Remove a setting value from the active profile."""

        settings = self._get_local_settings()
        settings.pop(self._validate_key(key), None)
        return gsettings_registry.get_registry().set_dict(
            self._SCHEMA,
            self._KEY,
            "a{sv}",
            settings,
            sub_path=self._sub_path(),
        )


class Extension:
    """Base class for Orca extensions."""

    GROUP_LABEL: ClassVar[str]
    DESCRIPTION: ClassVar[str] = ""
    VERSION: ClassVar[str] = ""
    AUTHOR: ClassVar[str] = ""
    ORGANIZATION: ClassVar[str] = ""
    COPYRIGHT: ClassVar[str] = ""
    WEBSITE: ClassVar[str] = ""

    def __init__(self) -> None:
        self._commands_initialized: bool = False
        self._is_user_extension: bool = False
        self._disabled: bool = False
        self._registered_command_names: list[str] = []
        self.module_name: str = type(self).__name__
        self.settings = ExtensionSettings(self.module_name)
        self.controller = dbus_service.get_remote_controller()
        msg = f"EXTENSION: {self.module_name} Registering D-Bus commands."
        debug.print_message(debug.LEVEL_INFO, msg, True)
        self.controller.register_decorated_module(self.module_name, self)

    def disable(self) -> None:
        """Disables this extension, preventing command registration."""

        self._disabled = True
        msg = f"EXTENSION: {self.module_name} has been disabled."
        debug.print_message(debug.LEVEL_INFO, msg, True)
        manager = command_manager.get_manager()
        manager.remove_commands(
            self._registered_command_names,
            f"disabled extension {self.module_name}",
        )
        self._registered_command_names.clear()
        self.controller.deregister_module_commands(self.module_name)

    def set_up_commands(self) -> None:
        """Sets up commands with CommandManager."""

        if self._disabled or self._commands_initialized:
            return
        self._commands_initialized = True
        self._register_commands()

    def _register_commands(self) -> None:
        """Registers the commands returned by _get_commands."""

        commands = self._get_commands()
        if not commands:
            msg = f"EXTENSION: {self.module_name} No commands to register."
            debug.print_message(debug.LEVEL_INFO, msg, True)
            return

        manager = command_manager.get_manager()
        for cmd in commands:
            if self._is_user_extension:
                cmd.set_function(self._wrap_function(cmd.get_function()))
                manager.add_user_extension_command(self.module_name, cmd)
            else:
                manager.add_command(cmd)
            self._registered_command_names.append(cmd.get_name())

        msg = f"EXTENSION: {self.module_name} {len(commands)} command(s) registered."
        debug.print_message(debug.LEVEL_INFO, msg, True)

    def mark_as_user_extension(self, settings_namespace: str | None = None) -> None:
        """Marks this extension as user-provided so commands get wrapped."""

        self._is_user_extension = True
        command_manager.get_manager().register_user_extension(self)
        if settings_namespace is not None:
            self.settings.set_namespace(settings_namespace)

    def _get_commands(self) -> list[Command]:
        """Override to provide commands for registration."""

        return []

    def get_preferences(self) -> list[ExtensionPreference]:
        """Override to provide declarative preferences."""

        return []

    def on_speech_output(  # pylint: disable=unused-argument
        self,
        output: SpeechOutput,
    ) -> SpeechOutputResult | None:
        """Called before Orca sends speech to the active speech provider."""

        return None

    def on_braille_output(  # pylint: disable=unused-argument
        self,
        output: BrailleOutput,
    ) -> BrailleOutputResult | None:
        """Called before Orca displays generated braille."""

        return None

    def on_shutdown(self) -> None:
        """Called when Orca is shutting down."""

    @staticmethod
    def _wrap_function(func):
        """Wraps a user extension method so it accepts and discards script and event."""

        def wrapper(_script, _event):
            return func()

        return wrapper



# Orca
#
# Copyright 2005-2008 Sun Microsystems Inc.
# Copyright 2011-2026 Igalia, S.L.
# Author: Joanmarie Diggs <jdiggs@igalia.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
# Boston MA  02110-1301 USA.

# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-arguments
# pylint: disable=too-many-positional-arguments

"""Manager for Orca profile creation, loading, and management."""

from __future__ import annotations

import subprocess
from typing import TYPE_CHECKING

from . import (
    command_manager,
    dbus_service,
    debug,
    gsettings_registry,
    guilabels,
    input_event,
    messages,
    orca,
    orca_modifier_manager,
    presentation_manager,
    profile_manager_command_definitions,
)
from .extension import Extension

if TYPE_CHECKING:
    from collections.abc import Callable

    from .command import Command
    from .profile_manager_preferences_grid import ProfilePreferencesGrid
    from .scripts import default


@gsettings_registry.get_registry().gsettings_schema(
    "org.gnome.Orca.ProfileMetadata",
    name="metadata",
)
class ProfileManager(Extension):
    """Manager for Orca profiles."""

    GROUP_LABEL = guilabels.GENERAL_PROFILES

    @gsettings_registry.get_registry().gsetting(
        key="display-name",
        schema="metadata",
        gtype="s",
        default="",
        summary="Original display name (label) of the profile or app",
    )
    def get_display_name(self) -> str:
        """Returns the display name for the active profile."""

        return gsettings_registry.get_registry().get_active_profile()

    @gsettings_registry.get_registry().gsetting(
        key="internal-name",
        schema="metadata",
        gtype="s",
        default="",
        summary="Original internal name (JSON dict key) of the profile",
    )
    def get_internal_name(self) -> str:
        """Returns the internal name for the active profile."""

        return gsettings_registry.get_registry().get_active_profile()

    def _get_commands(self) -> list[Command]:
        """Returns commands for registration."""

        return profile_manager_command_definitions.get_commands(self)

    @dbus_service.getter
    def get_available_profiles(self) -> list[list[str]]:
        """Returns list of available profiles as [display_name, internal_name] pairs."""

        profiles = self._get_stored_profiles(gsettings_registry.get_registry())
        for profile in profiles:
            if profile[1] == "default":
                profile[0] = guilabels.PROFILE_DEFAULT
                break
        return profiles

    @staticmethod
    def _get_stored_profiles(
        registry: gsettings_registry.GSettingsRegistry,
    ) -> list[list[str]]:
        """Returns available profiles by enumerating stored metadata."""

        try:
            result = subprocess.run(  # noqa: S603
                ["dconf", "list", gsettings_registry.GSETTINGS_PATH_PREFIX],  # noqa: S607
                capture_output=True,
                text=True,
                check=True,
            )
        except (subprocess.CalledProcessError, FileNotFoundError):
            return [["Default", "default"]]

        default_profile: list[str] | None = None
        profiles: list[list[str]] = []
        for entry in result.stdout.strip().split("\n"):
            if not entry.endswith("/"):
                continue
            sanitized_name = entry.rstrip("/")
            gs = registry.get_settings("metadata", sanitized_name)
            if gs is None:
                continue
            display_variant = gs.get_user_value("display-name")
            internal_variant = gs.get_user_value("internal-name")
            if display_variant is None or internal_variant is None:
                continue
            display_name = display_variant.get_string()
            internal_name = internal_variant.get_string()
            profile = [display_name, internal_name]
            if internal_name == "default":
                default_profile = profile
            else:
                profiles.append(profile)

        if default_profile is not None:
            profiles.insert(0, default_profile)
        elif not profiles:
            profiles.append(["Default", "default"])
        return profiles

    @dbus_service.getter
    def get_active_profile(self) -> str:
        """Returns the internal name of the currently active profile."""

        return gsettings_registry.get_registry().get_active_profile()

    @dbus_service.setter
    def set_active_profile(self, internal_name: str) -> bool:
        """Sets the active profile by internal name."""

        registry = gsettings_registry.get_registry()
        registry.clear_runtime_values()
        registry.set_active_profile(internal_name)
        return True

    def load_profile(self, internal_name: str) -> None:
        """Loads a profile by setting it active and reloading user settings."""

        msg = f"PROFILE MANAGER: Loading profile '{internal_name}'."
        debug.print_message(debug.LEVEL_INFO, msg, True)

        self.set_active_profile(internal_name)
        orca.load_user_settings(skip_reload_message=True)

    def create_profile(self, new_profile: list[str]) -> bool:
        """Create a new profile by copying the current active profile to dconf."""

        current_profile = self.get_active_profile()
        msg = f"PROFILE MANAGER: Creating profile '{new_profile[1]}' from '{current_profile}'."
        debug.print_message(debug.LEVEL_INFO, msg, True)

        registry = gsettings_registry.get_registry()
        old_profile = registry.sanitize_gsettings_path(current_profile)
        new_name = registry.sanitize_gsettings_path(new_profile[1])

        for schema_name in registry.get_schema_names():
            if schema_name == "voice":
                for voice_type in gsettings_registry.VOICE_TYPES:
                    sub = gsettings_registry.get_registry().voice_set_sub_path(voice_type)
                    old_gs = registry.get_settings("voice", old_profile, sub)
                    new_gs = registry.get_settings("voice", new_name, sub)
                    registry.copy_user_keys(old_gs, new_gs)
                continue
            old_gs = registry.get_settings(schema_name, old_profile)
            new_gs = registry.get_settings(schema_name, new_name)
            registry.copy_user_keys(old_gs, new_gs)

        metadata_gs = registry.get_settings("metadata", new_name)
        if metadata_gs is not None:
            metadata_gs.set_string("display-name", new_profile[0])
            metadata_gs.set_string("internal-name", new_profile[1])

        return True

    @dbus_service.getter
    def get_starting_profile(self) -> list[str]:
        """Returns the starting profile (always Default)."""

        return ["Default", "default"]

    @dbus_service.setter
    def set_starting_profile(self, _profile: list[str]) -> bool:
        """No-op for backwards compatibility. Starting profile is always Default."""

        return True

    def remove_profile(self, internal_name: str) -> None:
        """Removes a profile by internal name."""

        registry = gsettings_registry.get_registry()
        sanitized_name = registry.sanitize_gsettings_path(internal_name)
        path = f"{gsettings_registry.GSETTINGS_PATH_PREFIX}{sanitized_name}/"
        try:
            subprocess.run(  # noqa: S603 - dconf is a system dependency, not untrusted input
                ["dconf", "reset", "-f", path],  # noqa: S607 - full path would break across distros
                check=True,
            )
            msg = f"PROFILE MANAGER: Cleared GSettings for profile: {internal_name}"
            debug.print_message(debug.LEVEL_INFO, msg, True)
        except (subprocess.CalledProcessError, FileNotFoundError) as e:
            msg = f"PROFILE MANAGER: Failed to clear GSettings for profile: {internal_name}: {e}"
            debug.print_message(debug.LEVEL_INFO, msg, True)

    def rename_profile(self, old_internal_name: str, new_profile: list[str]) -> None:
        """Renames a profile."""

        gsettings_registry.get_registry().rename_profile(
            old_internal_name,
            new_profile[0],
            new_profile[1],
        )

    @dbus_service.command
    def cycle_settings_profile(
        self,
        script: default.Script | None = None,
        event: input_event.InputEvent | None = None,
        notify_user: bool = True,
    ) -> bool:
        """Cycle through the user's existing settings profiles."""

        tokens = [
            "PROFILE MANAGER: cycle_settings_profile. Script:",
            script,
            "Event:",
            event,
            "notify_user:",
            notify_user,
        ]
        debug.print_tokens(debug.LEVEL_INFO, tokens, True)

        profile_names = self.get_available_profiles()
        if not profile_names:
            if script is not None and notify_user:
                presentation_manager.get_manager().present_message(messages.PROFILE_NOT_FOUND)
            return True

        profiles = [(profile[0], profile[1]) for profile in profile_names]
        current_profile = self.get_active_profile()

        current_index = 0
        for i, (_, internal_name) in enumerate(profiles):
            if internal_name == current_profile:
                current_index = i
                break

        try:
            name, profile_id = profiles[current_index + 1]
        except IndexError:
            name, profile_id = profiles[0]

        self.set_active_profile(profile_id)

        orca_modifier_manager.get_manager().unset_orca_modifiers("Profile changing.")
        command_manager.get_manager().load_keyboard_layout()
        orca_modifier_manager.get_manager().refresh_orca_modifiers("Profile changed.")
        presentation_manager.get_manager().refresh_presenters()

        if script is not None:
            script.set_up_commands()
            if notify_user:
                presentation_manager.get_manager().present_message(
                    messages.PROFILE_CHANGED % name,
                    name,
                )

        return True

    @dbus_service.command
    def present_current_profile(
        self,
        script: default.Script | None = None,
        event: input_event.InputEvent | None = None,
        notify_user: bool = True,
    ) -> bool:
        """Present the name of the currently active profile."""

        tokens = [
            "PROFILE MANAGER: present_current_profile. Script:",
            script,
            "Event:",
            event,
            "notify_user:",
            notify_user,
        ]
        debug.print_tokens(debug.LEVEL_INFO, tokens, True)

        profile_names = self.get_available_profiles()
        current_profile_id = self.get_active_profile()

        name = current_profile_id
        for display_name, internal_name in profile_names:
            if internal_name == current_profile_id:
                name = display_name
                break

        if script is not None and notify_user:
            presentation_manager.get_manager().present_message(
                messages.PROFILE_CURRENT % name,
                name,
            )

        return True

    def create_preferences_grid(
        self,
        profile_loaded_callback: Callable[[list[str]], None],
        is_app_specific: bool = False,
        labels_update_callback: Callable[[], None] | None = None,
        unsaved_changes_checker: Callable[[], bool] | None = None,
    ) -> ProfilePreferencesGrid:
        """Returns the GtkGrid containing the profile management UI."""

        # pylint: disable-next=import-outside-toplevel
        from .profile_manager_preferences_grid import ProfilePreferencesGrid

        return ProfilePreferencesGrid(
            self,
            profile_loaded_callback,
            is_app_specific,
            labels_update_callback,
            unsaved_changes_checker,
        )


_manager = ProfileManager()


def get_manager() -> ProfileManager:
    """Returns the profile-manager singleton."""

    return _manager



# Orca
#
# Copyright 2026 Igalia, S.L.
# Author: Joanmarie Diggs <jdiggs@igalia.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
# Boston MA  02110-1301 USA.

"""Declarative preferences for user extensions."""

from __future__ import annotations

from dataclasses import dataclass
from enum import Enum
from typing import TYPE_CHECKING, Any

if TYPE_CHECKING:
    from collections.abc import Callable


class ExtensionPreferenceKind(Enum):
    """Supported user-extension preference control kinds."""

    INFO = "info"
    BOOLEAN = "boolean"
    STRING = "string"
    PATH = "path"
    COLOR = "color"
    INTEGER = "integer"
    FLOAT = "float"
    ENUM = "enum"
    STRING_LIST = "string-list"
    DICTIONARY = "dictionary"


@dataclass(frozen=True)
class ExtensionPreference:
    """Declarative description of one user-extension preference."""

    key: str
    label: str
    default: Any
    kind: ExtensionPreferenceKind
    minimum: int | float | None = None
    maximum: int | float | None = None
    options: tuple[tuple[Any, str], ...] = ()
    key_label: str = ""
    item_validator: Callable[[str], bool] | None = None
    item_error: str = ""
    value_label: str = ""
    value_validator: Callable[[str], bool] | None = None
    value_error: str = ""
    directory: bool = False

    @classmethod
    def info(cls, message: str) -> ExtensionPreference:
        """Return an informational text box for the generated dialog."""

        return cls("", message, None, ExtensionPreferenceKind.INFO)

    @classmethod
    def boolean(cls, key: str, label: str, default: bool = False) -> ExtensionPreference:
        """Return a boolean preference."""

        return cls(key, label, default, ExtensionPreferenceKind.BOOLEAN)

    @classmethod
    def string(cls, key: str, label: str, default: str = "") -> ExtensionPreference:
        """Return a string preference."""

        return cls(key, label, default, ExtensionPreferenceKind.STRING)

    @classmethod
    def path(
        cls,
        key: str,
        label: str,
        default: str = "",
        *,
        directory: bool = False,
    ) -> ExtensionPreference:
        """Return a file or directory path preference."""

        return cls(key, label, default, ExtensionPreferenceKind.PATH, directory=directory)

    @classmethod
    def color(cls, key: str, label: str, default: str = "#000000") -> ExtensionPreference:
        """Return a color preference stored as a hex string."""

        return cls(key, label, default, ExtensionPreferenceKind.COLOR)

    @classmethod
    def integer(
        cls,
        key: str,
        label: str,
        default: int = 0,
        minimum: int = 0,
        maximum: int = 100,
    ) -> ExtensionPreference:
        """Return an integer preference."""

        return cls(key, label, default, ExtensionPreferenceKind.INTEGER, minimum, maximum)

    @classmethod
    def floating(
        cls,
        key: str,
        label: str,
        default: float = 0.0,
        minimum: float = 0.0,
        maximum: float = 1.0,
    ) -> ExtensionPreference:
        """Return a floating-point preference."""

        return cls(key, label, default, ExtensionPreferenceKind.FLOAT, minimum, maximum)

    @classmethod
    def enum(
        cls,
        key: str,
        label: str,
        options: tuple[tuple[Any, str], ...],
        default: Any,
    ) -> ExtensionPreference:
        """Return an enumerated preference."""

        return cls(key, label, default, ExtensionPreferenceKind.ENUM, options=options)

    @classmethod
    def string_list(
        cls,
        key: str,
        label: str,
        default: list[str] | None = None,
        *,
        item_validator: Callable[[str], bool] | None = None,
        item_error: str = "",
    ) -> ExtensionPreference:
        """Return a string-list preference."""

        return cls(
            key,
            label,
            default or [],
            ExtensionPreferenceKind.STRING_LIST,
            item_validator=item_validator,
            item_error=item_error,
        )

    @classmethod
    def dictionary(
        cls,
        key: str,
        label: str,
        default: dict[str, bool | int | float | str] | None = None,
        *,
        key_label: str = "",
        value_label: str = "",
        key_validator: Callable[[str], bool] | None = None,
        value_validator: Callable[[str], bool] | None = None,
        key_error: str = "",
        value_error: str = "",
    ) -> ExtensionPreference:
        """Return a dictionary preference."""

        return cls(
            key,
            label,
            default or {},
            ExtensionPreferenceKind.DICTIONARY,
            key_label=key_label,
            item_validator=key_validator,
            item_error=key_error,
            value_label=value_label,
            value_validator=value_validator,
            value_error=value_error,
        )


GENERATED_DIALOG_KINDS = {
    ExtensionPreferenceKind.INFO,
    ExtensionPreferenceKind.BOOLEAN,
    ExtensionPreferenceKind.STRING,
    ExtensionPreferenceKind.PATH,
    ExtensionPreferenceKind.COLOR,
    ExtensionPreferenceKind.INTEGER,
    ExtensionPreferenceKind.FLOAT,
    ExtensionPreferenceKind.ENUM,
    ExtensionPreferenceKind.STRING_LIST,
    ExtensionPreferenceKind.DICTIONARY,
}


def is_supported_in_generated_dialog(preference: ExtensionPreference) -> bool:
    """Return True if Orca's generated dialog can display preference."""

    return preference.kind in GENERATED_DIALOG_KINDS



# Orca
#
# Copyright 2004-2009 Sun Microsystems Inc.
# Copyright 2026 Igalia, S.L.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
# Boston MA  02110-1301 USA.

# This file is auto-generated by tools/generate_braille_table_names.py.
# Do not edit manually. Re-run the tool after updating liblouis.

# pylint: disable=too-many-lines

"""Braille translation table names for localization.

These are in their own module so that we can present them in the correct
language when users change the synthesizer language on the fly without
having to reload a bunch of modules."""

from .orca_i18n import _  # pylint: disable=import-error

TABLE_NAMES: dict[str, str] = {
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "afr-za-g1.ctb": _("Afrikaans, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "afr-za-g2.ctb": _("Afrikaans, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "akk-borger.utb": _("Akkadian, Borger"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "akk.utb": _("Akkadian, U.S."),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ancient-languages-borger.utb": _("Ancient languages, Borger"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ancient-languages-us.utb": _("Ancient languages, U.S."),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ar-ar-comp8.utb": _("Arabic, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ar-ar-g1.utb": _("Arabic, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ar-ar-g2.ctb": _("Arabic, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "as-in-g1.utb": _("Assamese"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "aw-in-g1.utb": _("Awadhi"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ba.utb": _("Bashkir"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "be-in-g1.utb": _("Bengali"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "bel-comp.utb": _("Belarusian, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "bel-detailed.utb": _("Belarusian, with capitals"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "bel.utb": _("Belarusian"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "bg.ctb": _("Bulgarian, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "bg.utb": _("Bulgarian"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "bh.ctb": _("Bihari"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "bo.ctb": _("Tibetan, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "boxes.ctb": _("boxes"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "br-in-g1.utb": _("Braj"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ca-g1.ctb": _("Catalan"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "chr-us-g1.ctb": _("Cherokee"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ckb-g1.ctb": _("Kurdish"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "cop-eg-comp8.utb": _("Coptic, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "cop.utb": _("Coptic"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "cs-comp8.utb": _("Czech, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "cs-g1.ctb": _("Czech"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "cuneiform-transliterated-compact.utb": _("Cuneiform, transliterated, compact"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "cuneiform-transliterated.utb": _("Cuneiform, transliterated"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "cy-cy-g1.utb": _("Welsh, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "cy-cy-g2.ctb": _("Welsh, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "da-dk-g08.ctb": _("Danish, computer, 2022"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "da-dk-g08_1993.ctb": _("Danish, computer, 1993"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "da-dk-g16-lit_1993.ctb": _("Danish, uncontracted, 6-dot, 1993"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "da-dk-g16.ctb": _("Danish, uncontracted, 6-dot, 2022"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "da-dk-g16_1993.ctb": _("Danish, uncontracted, 6-dot, 1993"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "da-dk-g18.ctb": _("Danish, uncontracted, 8-dot, 2022"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "da-dk-g18_1993.ctb": _("Danish, uncontracted, 8-dot, 1993"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "da-dk-g26-lit_1993.ctb": _("Danish, contracted, 6-dot, 1993"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "da-dk-g26.ctb": _("Danish, contracted, 6-dot, 2022"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "da-dk-g26_1993.ctb": _("Danish, contracted, 6-dot, 1993"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "da-dk-g26l-lit_1993.ctb": _("Danish, partially contracted, 6-dot, 1993"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "da-dk-g26l_1993.ctb": _("Danish, partially contracted, 6-dot, 1993"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "da-dk-g28.ctb": _("Danish, contracted, 8-dot, 2022"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "da-dk-g28_1993.ctb": _("Danish, contracted, 8-dot, 1993"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "da-dk-g28l_1993.ctb": _("Danish, partially contracted, 8-dot, 1993"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "de-chess.ctb": _("de-chess"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "de-comp6.utb": _("German, computer, 6-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "de-de-comp8.ctb": _("German, computer, 8-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "de-g0-detailed.utb": _("German, uncontracted, with capitals"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "de-g0.utb": _("German, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "de-g1-detailed.ctb": _("German, partially contracted, with capitals"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "de-g1.ctb": _("German, partially contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "de-g2-detailed.ctb": _("German, contracted, with capitals"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "de-g2.ctb": _("German, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "dra.ctb": _("Dravidian, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "el.ctb": _("Greek"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en-chess.ctb": _("en-chess"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en-g3.ctb": _("English, contracted, grade 3"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en-gb-comp8.ctb": _("English, U.K., computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en-gb-g1.utb": _("English, U.K., uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en-GB-g2.ctb": _("English, U.K., contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en-in-g1.ctb": _("en-in-g1"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en-nabcc.utb": _("English, U.S., computer, NABCC"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en-ueb-g1.ctb": _("English, unified, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en-ueb-g2.ctb": _("English, unified, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en-ueb-math.ctb": _("en-ueb-math"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en-us-comp6.ctb": _("English, U.S., computer, 6-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en-us-comp8-ext.utb": _("English, U.S., computer, 8-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en-us-comp8.ctb": _("en-us-comp8"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en-us-g1.ctb": _("English, U.S., uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en-us-g2.ctb": _("English, U.S., contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en-us-interline.ctb": _("en-us-interline"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en-us-mathtext.ctb": _("en-us-mathtext"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "en_CA.ctb": _("English, Canada, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "eo-g1-x-system.ctb": _("Esperanto x-system"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "eo-g1.ctb": _("Esperanto"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "Es-Es-G0.utb": _("Spanish, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "es-g1.ctb": _("Spanish, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "es-g2.ctb": _("Spanish, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "es-no.utb": _("Spanish, Norway"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "et-6dot.utb": _("Estonian"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "et-g0.utb": _("Estonian, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ethio-g1.ctb": _("Ethiopic"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "fa-ir-comp8.ctb": _("Persian, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "fa-ir-g1.utb": _("Persian"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "fi-fi-8dot.ctb": _("Finnish, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "fi.utb": _("Finnish"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "fil-g2.ctb": _("Filipino, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "fr-bfu-comp6.utb": _("French, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "fr-bfu-comp8.utb": _("French, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "fr-bfu-g2.ctb": _("French, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ga-g1.utb": _("Irish, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ga-g2.ctb": _("Irish, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "gd.ctb": _("Scottish Gaelic, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "gon.ctb": _("Gondi"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "grc-international-en-composed.utb": _("Greek, international, English, composed"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "grc-international-en.utb": _("Greek, international, English, decomposed"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "grc-international-es.utb": _("Greek, international, Spanish"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "gu-in-g1.utb": _("Gujarati"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "haw-us-g1.ctb": _("Hawaiian"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "hbo-cantillated.utb": _("Hebrew, classical, cantillated"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "hbo-slim.utb": _("Hebrew, classical, slim"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "hbo.utb": _("Hebrew, classical, IHBC"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "he-IL-comp8.utb": _("Hebrew, modern, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "he-IL.utb": _("Hebrew, modern"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "hi-in-g1.utb": _("Hindi"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "hr-comp8.utb": _("Croatian, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "hr-g1.ctb": _("Croatian"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "hu-hu-comp8.ctb": _("Hungarian, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "hu-hu-g1.ctb": _("Hungarian, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "hu-hu-g2.ctb": _("Hungarian, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "hy.ctb": _("Armenian, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "IPA.utb": _("International Phonetic Alphabet"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "is.ctb": _("is"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "it-it-comp6.utb": _("Italian"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "it-it-comp8.utb": _("Italian, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "iu-ca-g1.ctb": _("Inuktitut"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ja-kantenji-ucs2.utb": _("Japanese, Kantenji"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ja-kantenji.utb": _("Japanese, Kantenji"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ja-rokutenkanji.utb": _("Japanese, Rokuten Kanji"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ka-in-g1.utb": _("Kannada"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ka.utb": _("Georgian"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "kh-in-g1.utb": _("Khasi"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "kk.utb": _("Kazakh"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "km-g1.utb": _("Khmer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ko-2006-g1.ctb": _("Korean, uncontracted, 2006"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ko-2006-g2.ctb": _("Korean, contracted, 2006"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ko-g1.ctb": _("Korean, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ko-g2.ctb": _("Korean, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "kok.ctb": _("Konkani"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "kru.ctb": _("Kurukh"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ks-in-g1.utb": _("ks-in-g1"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "lg-ug-g1.utb": _("Luganda"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "lo-g1.utb": _("Lao"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "lt-6dot.utb": _("Lithuanian, 6-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "lt-8dot.utb": _("Lithuanian, 8-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "lt.ctb": _("lt"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "Lv-Lv-g1.utb": _("Latvian"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "mao-nz-g1.ctb": _("Maori"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "mk-g1.utb": _("Macedonian"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ml-in-g1.utb": _("Malayalam"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "mn-in-g1.utb": _("Manipuri"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "mn-MN-g1.utb": _("Mongolian, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "mn-MN-g2.ctb": _("Mongolian, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "mr-in-g1.utb": _("Marathi"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ms-my-g2.ctb": _("Malay"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "mt.ctb": _("Maltese, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "mun.ctb": _("Munda"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "mwr.ctb": _("Marwari"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "my-g1.utb": _("Burmese, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "my-g2.ctb": _("Burmese, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ne.ctb": _("ne"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "nl-comp8.utb": _("Dutch, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "no-no-8dot-fallback-6dot-g0.utb": _("Norwegian, uncontracted, 8-dot, 6-dot fallback"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "no-no-8dot.utb": _("Norwegian, uncontracted, 8-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "no-no-comp8.ctb": _("Norwegian, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "no-no-g0.utb": _("Norwegian, uncontracted, 6-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "no-no-g1.ctb": _("Norwegian, contracted, grade 1"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "no-no-g2.ctb": _("Norwegian, contracted, grade 2"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "no-no-g3.ctb": _("Norwegian, contracted, grade 3"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "no-no-generic.ctb": _("no-no-generic"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "np-in-g1.utb": _("Nepali"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "nso-za-g1.utb": _("Sepedi, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "nso-za-g2.ctb": _("Sepedi, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ny-mw.utb": _("Chichewa"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "or-in-g1.utb": _("Oriya"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ovd-6g0.utb": _("Elfdalian, 6-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ovd-8g0.utb": _("Elfdalian, 8-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "pi.ctb": _("Pali"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "pl-pl-comp8.ctb": _("Polish, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "Pl-Pl-g1.utb": _("Polish"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "pt-comp6.utb": _("Portuguese, computer, 6-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "pt-pt-comp8.ctb": _("Portuguese, computer, 8-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "pt-pt-g1.utb": _("Portuguese, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "pt-pt-g2.ctb": _("Portuguese, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "pu-in-g1.utb": _("Punjabi"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ro-g0.utb": _("Romanian, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ro.ctb": _("Romanian, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ru-comp6.utb": _("Russian, computer, 6-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ru-comp8.utb": _("Russian, computer, 8-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ru-litbrl-detailed.utb": _("Russian, with capitals"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ru-litbrl.ctb": _("Russian"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ru-ru-g1.ctb": _("Russian, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "rw-rw-g1.utb": _("Kinyarwanda"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sa-in-g1.utb": _("Sanskrit"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sah.utb": _("Yakut"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "se-se.ctb": _("se-se"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "si-in-g1.utb": _("Sindhi"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sin.utb": _("sin"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sk-g1.ctb": _("Slovak"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sk-sk-g1.utb": _("sk-sk-g1"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sk-sk.utb": _("sk-sk"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sl-si-comp8.ctb": _("Slovenian, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sl-si-g1.utb": _("Slovenian"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "smi-6g0.utb": _("Sami, 6-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "smi-8g0.utb": _("Sami, 8-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sot-za-g1.ctb": _("Sesotho, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sot-za-g2.ctb": _("Sesotho, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sr-Cyrl.ctb": _("Serbian, Cyrillic"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sr-g1.ctb": _("Serbian"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sv-1989.ctb": _("Swedish, computer, 1989"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sv-1996.ctb": _("Swedish, computer, 1996"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sv-6g0d.utb": _("Swedish, uncontracted, 6-dot, detailed"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sv-6g0p.utb": _("Swedish, uncontracted, 6-dot, phonetics"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sv-6g1d.ctb": _("Swedish, partially contracted, 6-dot, detailed"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sv-6g2d.ctb": _("Swedish, contracted, 6-dot, detailed"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sv-8g0d.utb": _("Swedish, uncontracted, 8-dot, detailed"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sv-8g0n.utb": _("Swedish, uncontracted, 8-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sv-8g1d.ctb": _("Swedish, partially contracted, 8-dot, detailed"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sv-8g1n.ctb": _("Swedish, partially contracted, 8-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sv-8g2d.ctb": _("Swedish, contracted, 8-dot, detailed"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sv-8g2n.ctb": _("Swedish, contracted, 8-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sv-g0.utb": _("Swedish, uncontracted, 6-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sv-g1.ctb": _("Swedish, partially contracted, 6-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sv-g2.ctb": _("Swedish, contracted, 6-dot"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sw-ke-g1-2.ctb": _("Swahili, contracted, grade 1.2"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sw-ke-g1-3.ctb": _("Swahili, contracted, grade 1.3"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sw-ke-g1-4.ctb": _("Swahili, contracted, grade 1.4"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sw-ke-g1-5.ctb": _("Swahili, contracted, grade 1.5"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sw-ke-g1.utb": _("Swahili, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "sw-ke-g2.ctb": _("Swahili, contracted, grade 2"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "syc.utb": _("Syriac"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ta-ta-g1.ctb": _("Tamil"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ta.ctb": _("Tamil, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "te-in-g1.utb": _("Telugu"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "th-comp8-backward.utb": _("Thai, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "th-g0.utb": _("Thai, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "th-g1.utb": _("Thai, partially contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "th-g2.ctb": _("Thai, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "tr-g1.ctb": _("Turkish, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "tr-g2.ctb": _("Turkish, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "tr.ctb": _("Turkish, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "tsn-za-g1.ctb": _("Setswana, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "tsn-za-g2.ctb": _("Setswana, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "tt.utb": _("Tatar"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "uga.utb": _("Ugaritic"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "uk-comp.utb": _("Ukrainian, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "uk-detailed.utb": _("Ukrainian, with capitals"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "uk.utb": _("Ukrainian"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "unicode-braille.utb": _("unicode-braille"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ur-pk-g1.utb": _("Urdu, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ur-pk-g2.ctb": _("Urdu, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "uz-g1.utb": _("Uzbek"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ve-za-g1.utb": _("Tshivenda, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "ve-za-g2.ctb": _("Tshivenda, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "vi-cb8.utb": _("Vietnamese, computer"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "vi-saigon-g1.ctb": _("Vietnamese, Saigon, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "vi-vn-g0.utb": _("Vietnamese, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "vi-vn-g1.ctb": _("Vietnamese, partially contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "vi-vn-g2.ctb": _("Vietnamese, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "xh-za-g1.utb": _("isiXhosa, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "xh-za-g2.ctb": _("isiXhosa, contracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "yi.utb": _("Yiddish"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "zh-chn.ctb": _("Mandarin, mainland China, without tones"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "zh-hk.ctb": _("Cantonese"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "zh-tw.ctb": _("Mandarin, Taiwan, bopomofo"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "zhcn-cbs.ctb": _("Mandarin, mainland China, common braille scheme"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "zhcn-g1.ctb": _("Mandarin, mainland China, with tones"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "zhcn-g2.ctb": _("Mandarin, mainland China, double-phonic"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "zu-za-g1.utb": _("isiZulu, uncontracted"),
    # Translators: This is the name of a braille translation table
    # provided by liblouis (https://liblouis.io). Many of the terms
    # used in these names (e.g. contracted, uncontracted, grade,
    # computer braille) are explained at
    # https://en.wikipedia.org/wiki/Braille.
    "zu-za-g2.ctb": _("isiZulu, contracted"),
}



# Orca
#
# Copyright (C) 2014 Igalia, S.L.
#
# Author: Joanmarie Diggs <jdiggs@igalia.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
# Boston MA  02110-1301 USA.

"""Custom script utilities for gnome-shell."""

from __future__ import annotations

from typing import TYPE_CHECKING

from orca import debug, script_utilities
from orca.ax_text import AXText

if TYPE_CHECKING:
    import gi

    gi.require_version("Atspi", "2.0")
    from gi.repository import Atspi


class Utilities(script_utilities.Utilities):
    """Custom script utilities for gnome-shell."""

    def inserted_text(self, event: Atspi.Event) -> str:
        if event.any_data:
            return event.any_data

        # https://gitlab.gnome.org/GNOME/mutter/-/issues/3826
        if event.detail1 == -1:
            msg = "GNOME SHELL: Broken text insertion event"
            debug.print_message(debug.LEVEL_INFO, msg, True)

            string = AXText.get_all_text(event.source)
            if string:
                msg = f"GNOME SHELL: Returning last char in '{string}'"
                debug.print_message(debug.LEVEL_INFO, msg, True)
                return string[-1]

            msg = "GNOME SHELL: Unable to correct broken text insertion event"
            debug.print_message(debug.LEVEL_INFO, msg, True)

        return ""



# Orca
#
# Copyright 2024 Igalia, S.L.
# Copyright 2024 GNOME Foundation Inc.
# Author: Joanmarie Diggs <jdiggs@igalia.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
# Boston MA  02110-1301 USA.

"""Wrapper for the Atspi.Document interface."""

from __future__ import annotations

import gi

gi.require_version("Atspi", "2.0")
from gi.repository import Atspi, GLib

from . import debug
from .ax_object import AXObject


class AXDocument:
    """Wrapper for the Atspi.Document interface."""

    @staticmethod
    def get_page_count(document: Atspi.Accessible) -> int:
        """Returns the page count of document."""

        if not AXObject.supports_document(document):
            return 0

        try:
            count = Atspi.Document.get_page_count(document)
        except GLib.GError as error:
            msg = f"AXDocument: Exception in get_page_count: {error}"
            debug.print_message(debug.LEVEL_INFO, msg, True)
            return 0

        tokens = ["AXDocument: Page count of", document, f"is {count}"]
        debug.print_tokens(debug.LEVEL_INFO, tokens, True)
        return count

    @staticmethod
    def get_locale(document: Atspi.Accessible) -> str:
        """Returns the locale of document."""

        if not AXObject.supports_document(document):
            return ""

        try:
            result = Atspi.Document.get_locale(document)
        except GLib.GError as error:
            msg = f"AXDocument: Exception in get_locale: {error}"
            debug.print_message(debug.LEVEL_INFO, msg, True)
            return ""

        if result is None:
            tokens = ["AXDocument: get_locale failed for", document]
            debug.print_tokens(debug.LEVEL_INFO, tokens, True)
            return ""

        tokens = ["AXDocument: Locale of", document, f"is '{result}'"]
        debug.print_tokens(debug.LEVEL_INFO, tokens, True)
        return result

    @staticmethod
    def get_attributes_dict(document: Atspi.Accessible) -> dict[str, str]:
        """Returns a dict with the document-attributes of document."""

        if not AXObject.supports_document(document):
            return {}

        try:
            result = Atspi.Document.get_document_attributes(document)
        except GLib.GError as error:
            msg = f"AXDocument: Exception in get_attributes_dict: {error}"
            debug.print_message(debug.LEVEL_INFO, msg, True)
            return {}

        tokens = ["AXDocument: Attributes of", document, "are:", result]
        debug.print_tokens(debug.LEVEL_INFO, tokens, True)
        return result or {}



# Orca
#
# Copyright 2026 Igalia, S.L.
# Author: Joanmarie Diggs <jdiggs@igalia.com>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
# Boston MA  02110-1301 USA.

"""Helpers for working with locale and language codes."""

from __future__ import annotations

import locale

try:
    from babel import Locale as _BabelLocale
    from babel.core import UnknownLocaleError as _UnknownLocaleError
except ImportError:
    _BabelLocale = None  # type: ignore[assignment, misc]
    _UnknownLocaleError = None  # type: ignore[assignment, misc]


def get_current_language_and_dialect(category: int = locale.LC_MESSAGES) -> tuple[str, str]:
    """Returns the (language, dialect) of the user's current locale, or empty strings."""

    value = locale.getlocale(category)[0] or ""
    if value in ("", "C"):
        return "", ""
    parts = value.split("_")
    return parts[0], parts[1] if len(parts) > 1 else ""


def get_current_language(category: int = locale.LC_MESSAGES) -> str:
    """Returns the language of the user's current locale, or an empty string."""

    return get_current_language_and_dialect(category)[0]


def get_known_language_codes() -> list[tuple[str, str]]:
    """Returns deduplicated (language, dialect) pairs for the locales known to the system."""

    seen: set[tuple[str, str]] = set()
    result: list[tuple[str, str]] = []
    for alias in locale.locale_alias:
        stripped = alias.split(".")[0].split("@")[0]
        parts = stripped.split("_")
        lang = parts[0].lower()
        if len(lang) != 2 or not lang.isalpha():
            continue
        region = parts[1] if len(parts) > 1 and len(parts[1]) >= 2 else ""
        candidates = [(lang, "")]
        if region:
            candidates.append((lang, region))
        for candidate in candidates:
            if candidate not in seen:
                seen.add(candidate)
                result.append(candidate)
    return result


def get_language_display_name(lang: str, dialect: str = "", in_own_language: bool = False) -> str:
    """Returns a human-readable display name for a language code."""

    if not dialect and "-" in lang:
        lang, dialect = lang.split("-", 1)

    if _BabelLocale is not None:
        try:
            locale_id = f"{lang}_{dialect}" if dialect else lang
            target = _BabelLocale.parse(locale_id)
            display_locale = target if in_own_language else _BabelLocale.default()
            if display := target.get_display_name(display_locale):
                if dialect:
                    base_display = _BabelLocale.parse(lang).get_display_name(display_locale)
                    if display == base_display:
                        return f"{display} ({dialect})"
                return display
        except (ValueError, _UnknownLocaleError):
            try:
                base_locale = _BabelLocale.parse(lang)
                display_locale = base_locale if in_own_language else _BabelLocale.default()
                if dialect and (base := base_locale.get_display_name(display_locale)):
                    return f"{base} ({dialect})"
            except (ValueError, _UnknownLocaleError):
                pass
    return f"{lang}-{dialect}" if dialect else lang


def is_standard_locale(lang: str, dialect: str) -> bool:
    """Returns True if babel recognizes the lang+dialect as a distinct locale."""

    if _BabelLocale is None:
        return len(dialect) <= 3 and dialect.isalpha()
    try:
        full = _BabelLocale.parse(f"{lang}_{dialect}")
        base = _BabelLocale.parse(lang)
        return full.get_display_name() != base.get_display_name()
    except (ValueError, _UnknownLocaleError):
        return False



# Orca
#
# Copyright 2004-2009 Sun Microsystems Inc.
# Copyright 2011-2026 Igalia, S.L.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., Franklin Street, Fifth Floor,
# Boston MA  02110-1301 USA.

"""Command definitions for speech presentation."""

from __future__ import annotations

from typing import TYPE_CHECKING

from . import cmdnames, keybindings
from .command import Command, KeyboardCommand

if TYPE_CHECKING:
    from .speech_presenter import SpeechPresenter


def get_commands(owner: SpeechPresenter) -> list[Command]:
    """Returns commands for speech presentation."""

    kb_orca_v = keybindings.KeyBinding("v", keybindings.ORCA_MODIFIER_MASK)
    kb_orca_f11 = keybindings.KeyBinding("F11", keybindings.ORCA_MODIFIER_MASK)
    kb_orca_shift_d = keybindings.KeyBinding("d", keybindings.ORCA_SHIFT_MODIFIER_MASK)

    return [
        KeyboardCommand(
            "changeNumberStyleHandler",
            owner.change_number_style,
            owner.GROUP_LABEL,
            cmdnames.CHANGE_NUMBER_STYLE,
        ),
        KeyboardCommand(
            "toggleSpeechVerbosityHandler",
            owner.toggle_verbosity,
            owner.GROUP_LABEL,
            cmdnames.TOGGLE_SPEECH_VERBOSITY,
            desktop_keybinding=kb_orca_v,
            laptop_keybinding=kb_orca_v,
        ),
        KeyboardCommand(
            "toggleSpeakingIndentationHandler",
            owner.toggle_indentation,
            owner.GROUP_LABEL,
            cmdnames.TOGGLE_SPOKEN_INDENTATION,
        ),
        KeyboardCommand(
            "toggleTableCellReadModeHandler",
            owner.toggle_table_cell_reading_mode,
            owner.GROUP_LABEL,
            cmdnames.TOGGLE_TABLE_CELL_READ_MODE,
            desktop_keybinding=kb_orca_f11,
            laptop_keybinding=kb_orca_f11,
        ),
        KeyboardCommand(
            "toggle_speech_monitor",
            owner.toggle_monitor,
            owner.GROUP_LABEL,
            cmdnames.TOGGLE_SPEECH_MONITOR,
            desktop_keybinding=kb_orca_shift_d,
            laptop_keybinding=kb_orca_shift_d,
        ),
        KeyboardCommand(
            "cycle_text_attribute_change_mode",
            owner.cycle_text_attribute_change_mode,
            owner.GROUP_LABEL,
            cmdnames.CYCLE_TEXT_ATTRIBUTE_CHANGE_MODE,
        ),
    ]
